@@ -30,19 +30,17 @@ where
 
 
 import Citeproc.Types
-import Citeproc.Locale (lookupQuotes)
 import Citeproc.CaseTransform
 import Data.Ord ()
-import Data.Char (isAlphaNum, isSpace, isAscii, isPunctuation)
+import qualified Data.Map as M
+import Data.Char (isAlphaNum, isSpace, isPunctuation)
 import Data.Text (Text)
 import qualified Data.Text as T
 import Data.Foldable (fold)
 import Data.Functor.Identity
-import Data.Attoparsec.Text as P
 import Data.Aeson (FromJSON(..), ToJSON(..), Value(..), object)
 import Control.Monad.Trans.State
-import Control.Monad (guard, when)
-import Control.Applicative ((<|>))
+import Control.Monad (when)
 import Data.Generics.Uniplate.Direct
 
 data CslJson a =
@@ -103,7 +101,9 @@ instance Biplate (CslJson a) (CslJson a) where
 
 instance CiteprocOutput (CslJson Text) where
   toText                = fold
-  fromText              = parseCslJson mempty
+  fromText              = \t -> if T.null t
+                                   then CslEmpty
+                                   else CslText t
   dropTextWhile         = dropTextWhile'
   dropTextWhileEnd      = dropTextWhileEnd'
   addFontVariant x      =
@@ -183,106 +183,6 @@ dropTextWhileEnd' f el =
      CslDiv t x -> CslDiv t (dropTextWhileEnd' f x)
      CslLink t x -> CslLink t (dropTextWhileEnd' f x)
 
-parseCslJson :: Locale -> Text -> CslJson Text
-parseCslJson locale t =
-  case P.parseOnly
-         (P.many' (pCslJson locale) <* P.endOfInput) t of
-    Left _   -> CslText t
-    Right xs -> mconcat xs
-
-pCslJson :: Locale -> P.Parser (CslJson Text)
-pCslJson locale = P.choice
-  [ pCslText
-  , pCslQuoted
-  , pCslItalic
-  , pCslBold
-  , pCslUnderline
-  , pCslNoDecoration
-  , pCslSmallCaps
-  , pCslSup
-  , pCslSub
-  , pCslBaseline
-  , pCslNoCase
-  , pCslSymbol
-  ]
- where
-  ((outerOpenQuote, outerCloseQuote), (innerOpenQuote, innerCloseQuote)) =
-     lookupQuotes locale
-  isSpecialChar c = c == '<' || c == '>' || c == '\'' || c == '"' ||
-       c == '’' || (not (isAscii c) && (isSuperscriptChar c || isQuoteChar c))
-  isQuoteChar = P.inClass
-       (T.unpack (outerOpenQuote <> outerCloseQuote <>
-                 innerOpenQuote <> innerCloseQuote))
-  isSuperscriptChar = P.inClass superscriptChars
-  isApostrophe '\'' = True
-  isApostrophe '’'  = True
-  isApostrophe _    = False
-  pCsl = pCslJson locale
-  notFollowedBySpace =
-    P.peekChar' >>= guard . not . isSpaceChar
-  isSpaceChar = P.inClass [' ','\t','\n','\r']
-  pOpenQuote = (("\"" <$ P.char '"')
-                <|> ("'" <$ P.char '\'')
-                <|> (outerCloseQuote <$ P.string outerOpenQuote)
-                <|> (innerCloseQuote <$ P.string innerOpenQuote))
-                 <* notFollowedBySpace
-  pSpace = P.skipWhile isSpaceChar
-  pCslText = CslText . addNarrowSpace <$>
-    (  do t <- P.takeWhile1 (\c -> isAlphaNum c && not (isSpecialChar c))
-          -- apostrophe
-          P.option t $ do _ <- P.satisfy isApostrophe
-                          t' <- P.takeWhile1 isAlphaNum
-                          return (t <> "’" <> t')
-    <|>
-      (P.takeWhile1 (\c -> not (isAlphaNum c || isSpecialChar c))) )
-  pCslQuoted = CslQuoted <$>
-    do cl <- pOpenQuote
-       mbc <- peekChar
-       case mbc of
-         Just c  | T.singleton c == cl -> fail "unexpected close quote"
-         _ -> return ()
-       mconcat <$> P.manyTill' pCsl (P.string cl)
-  pCslSymbol = do
-    c <- P.satisfy isSpecialChar
-    return $
-       if isApostrophe c
-          then CslText "’"
-          else charToSup c
-  pCslItalic = CslItalic . mconcat <$>
-    (P.string "<i>" *> P.manyTill' pCsl (P.string "</i>"))
-  pCslBold = CslBold . mconcat <$>
-    (P.string "<b>" *> P.manyTill' pCsl (P.string "</b>"))
-  pCslUnderline = CslUnderline . mconcat <$>
-    (P.string "<u>" *> P.manyTill' pCsl (P.string "</u>"))
-  pCslNoDecoration = CslNoDecoration . mconcat <$>
-    (P.string "<span" *> pSpace *>
-     P.string "class=\"nodecor\"" *> pSpace *> P.char '>' *>
-     P.manyTill' pCsl (P.string "</span>"))
-  pCslSup = CslSup . mconcat <$>
-    (P.string "<sup>" *> P.manyTill' pCsl (P.string "</sup>"))
-  pCslSub = CslSub . mconcat <$>
-    (P.string "<sub>" *> P.manyTill' pCsl (P.string "</sub>"))
-  pCslBaseline = CslBaseline . mconcat <$>
-    (P.string "<span" *> pSpace *> P.string "style=\"baseline\">" *>
-      P.manyTill' pCsl (P.string "</span>"))
-  pCslSmallCaps = CslSmallCaps . mconcat <$>
-    ((P.string "<span" *> pSpace *>
-      P.string "style=\"font-variant:" *> pSpace *>
-      P.string "small-caps;" *> pSpace *> P.char '"' *>
-      pSpace *> P.char '>' *> P.manyTill' pCsl (P.string "</span>"))
-    <|>
-     (P.string "<sc>" *> P.manyTill' pCsl (P.string "</sc>")))
-  pCslNoCase = CslNoCase . mconcat <$>
-    (P.string "<span" *> pSpace *>
-     P.string "class=\"nocase\"" *> pSpace *> P.char '>' *>
-     P.manyTill' pCsl (P.string "</span>"))
-  addNarrowSpace =
-    T.replace " ;" "\x202F;" .
-    T.replace " ?" "\x202F?" .
-    T.replace " !" "\x202F!" .
-    T.replace " »" "\x202F»" .
-    T.replace "« " "«\x202F"
-
 data RenderContext =
   RenderContext
   { useOuterQuotes  :: Bool
@@ -305,7 +205,11 @@ renderCslJson useEntities locale =
   go :: RenderContext -> CslJson Text -> Text
   go ctx el =
     case el of
-      CslText t -> escape t
+      CslText t
+        | Just (c, "") <- T.uncons t
+        , Just t' <- M.lookup c superscriptChars
+          -> "<sup>" <> t' <> "</sup>"
+        | otherwise -> escape t
       CslEmpty -> mempty
       CslConcat x y -> go ctx x <> go ctx y
       CslQuoted x
@@ -611,219 +515,3 @@ punctuationInsideQuotes = go
       CslText t                   -> CslText t
       CslEmpty                    -> CslEmpty
 
-superscriptChars :: [Char]
-superscriptChars =
-  [ '\x00AA'
-  , '\x00B2'
-  , '\x00B3'
-  , '\x00B9'
-  , '\x00BA'
-  , '\x02B0'
-  , '\x02B1'
-  , '\x02B2'
-  , '\x02B3'
-  , '\x02B4'
-  , '\x02B5'
-  , '\x02B6'
-  , '\x02B7'
-  , '\x02B8'
-  , '\x02E0'
-  , '\x02E1'
-  , '\x02E2'
-  , '\x02E3'
-  , '\x02E4'
-  , '\x1D2C'
-  , '\x1D2D'
-  , '\x1D2E'
-  , '\x1D30'
-  , '\x1D31'
-  , '\x1D32'
-  , '\x1D33'
-  , '\x1D34'
-  , '\x1D35'
-  , '\x1D36'
-  , '\x1D37'
-  , '\x1D38'
-  , '\x1D39'
-  , '\x1D3A'
-  , '\x1D3C'
-  , '\x1D3D'
-  , '\x1D3E'
-  , '\x1D3F'
-  , '\x1D40'
-  , '\x1D41'
-  , '\x1D42'
-  , '\x1D43'
-  , '\x1D44'
-  , '\x1D45'
-  , '\x1D46'
-  , '\x1D47'
-  , '\x1D48'
-  , '\x1D49'
-  , '\x1D4A'
-  , '\x1D4B'
-  , '\x1D4C'
-  , '\x1D4D'
-  , '\x1D4F'
-  , '\x1D50'
-  , '\x1D51'
-  , '\x1D52'
-  , '\x1D53'
-  , '\x1D54'
-  , '\x1D55'
-  , '\x1D56'
-  , '\x1D57'
-  , '\x1D58'
-  , '\x1D59'
-  , '\x1D5A'
-  , '\x1D5B'
-  , '\x1D5C'
-  , '\x1D5D'
-  , '\x1D5E'
-  , '\x1D5F'
-  , '\x1D60'
-  , '\x1D61'
-  , '\x2070'
-  , '\x2071'
-  , '\x2074'
-  , '\x2075'
-  , '\x2076'
-  , '\x2077'
-  , '\x2078'
-  , '\x2079'
-  , '\x207A'
-  , '\x207B'
-  , '\x207C'
-  , '\x207D'
-  , '\x207E'
-  , '\x207F'
-  , '\x2120'
-  , '\x2122'
-  , '\x3192'
-  , '\x3193'
-  , '\x3194'
-  , '\x3195'
-  , '\x3196'
-  , '\x3197'
-  , '\x3198'
-  , '\x3199'
-  , '\x319A'
-  , '\x319B'
-  , '\x319C'
-  , '\x319D'
-  , '\x319E'
-  , '\x319F'
-  , '\x02C0'
-  , '\x02C1'
-  , '\x06E5'
-  , '\x06E6'
-  ]
-
-charToSup :: Char -> CslJson Text
-charToSup c =
-  case c of
-    '\x00AA' -> CslSup (CslText "\x0061")
-    '\x00B2' -> CslSup (CslText "\x0032")
-    '\x00B3' -> CslSup (CslText "\x0033")
-    '\x00B9' -> CslSup (CslText "\x0031")
-    '\x00BA' -> CslSup (CslText "\x006F")
-    '\x02B0' -> CslSup (CslText "\x0068")
-    '\x02B1' -> CslSup (CslText "\x0266")
-    '\x02B2' -> CslSup (CslText "\x006A")
-    '\x02B3' -> CslSup (CslText "\x0072")
-    '\x02B4' -> CslSup (CslText "\x0279")
-    '\x02B5' -> CslSup (CslText "\x027B")
-    '\x02B6' -> CslSup (CslText "\x0281")
-    '\x02B7' -> CslSup (CslText "\x0077")
-    '\x02B8' -> CslSup (CslText "\x0079")
-    '\x02E0' -> CslSup (CslText "\x0263")
-    '\x02E1' -> CslSup (CslText "\x006C")
-    '\x02E2' -> CslSup (CslText "\x0073")
-    '\x02E3' -> CslSup (CslText "\x0078")
-    '\x02E4' -> CslSup (CslText "\x0295")
-    '\x1D2C' -> CslSup (CslText "\x0041")
-    '\x1D2D' -> CslSup (CslText "\x00C6")
-    '\x1D2E' -> CslSup (CslText "\x0042")
-    '\x1D30' -> CslSup (CslText "\x0044")
-    '\x1D31' -> CslSup (CslText "\x0045")
-    '\x1D32' -> CslSup (CslText "\x018E")
-    '\x1D33' -> CslSup (CslText "\x0047")
-    '\x1D34' -> CslSup (CslText "\x0048")
-    '\x1D35' -> CslSup (CslText "\x0049")
-    '\x1D36' -> CslSup (CslText "\x004A")
-    '\x1D37' -> CslSup (CslText "\x004B")
-    '\x1D38' -> CslSup (CslText "\x004C")
-    '\x1D39' -> CslSup (CslText "\x004D")
-    '\x1D3A' -> CslSup (CslText "\x004E")
-    '\x1D3C' -> CslSup (CslText "\x004F")
-    '\x1D3D' -> CslSup (CslText "\x0222")
-    '\x1D3E' -> CslSup (CslText "\x0050")
-    '\x1D3F' -> CslSup (CslText "\x0052")
-    '\x1D40' -> CslSup (CslText "\x0054")
-    '\x1D41' -> CslSup (CslText "\x0055")
-    '\x1D42' -> CslSup (CslText "\x0057")
-    '\x1D43' -> CslSup (CslText "\x0061")
-    '\x1D44' -> CslSup (CslText "\x0250")
-    '\x1D45' -> CslSup (CslText "\x0251")
-    '\x1D46' -> CslSup (CslText "\x1D02")
-    '\x1D47' -> CslSup (CslText "\x0062")
-    '\x1D48' -> CslSup (CslText "\x0064")
-    '\x1D49' -> CslSup (CslText "\x0065")
-    '\x1D4A' -> CslSup (CslText "\x0259")
-    '\x1D4B' -> CslSup (CslText "\x025B")
-    '\x1D4C' -> CslSup (CslText "\x025C")
-    '\x1D4D' -> CslSup (CslText "\x0067")
-    '\x1D4F' -> CslSup (CslText "\x006B")
-    '\x1D50' -> CslSup (CslText "\x006D")
-    '\x1D51' -> CslSup (CslText "\x014B")
-    '\x1D52' -> CslSup (CslText "\x006F")
-    '\x1D53' -> CslSup (CslText "\x0254")
-    '\x1D54' -> CslSup (CslText "\x1D16")
-    '\x1D55' -> CslSup (CslText "\x1D17")
-    '\x1D56' -> CslSup (CslText "\x0070")
-    '\x1D57' -> CslSup (CslText "\x0074")
-    '\x1D58' -> CslSup (CslText "\x0075")
-    '\x1D59' -> CslSup (CslText "\x1D1D")
-    '\x1D5A' -> CslSup (CslText "\x026F")
-    '\x1D5B' -> CslSup (CslText "\x0076")
-    '\x1D5C' -> CslSup (CslText "\x1D25")
-    '\x1D5D' -> CslSup (CslText "\x03B2")
-    '\x1D5E' -> CslSup (CslText "\x03B3")
-    '\x1D5F' -> CslSup (CslText "\x03B4")
-    '\x1D60' -> CslSup (CslText "\x03C6")
-    '\x1D61' -> CslSup (CslText "\x03C7")
-    '\x2070' -> CslSup (CslText "\x0030")
-    '\x2071' -> CslSup (CslText "\x0069")
-    '\x2074' -> CslSup (CslText "\x0034")
-    '\x2075' -> CslSup (CslText "\x0035")
-    '\x2076' -> CslSup (CslText "\x0036")
-    '\x2077' -> CslSup (CslText "\x0037")
-    '\x2078' -> CslSup (CslText "\x0038")
-    '\x2079' -> CslSup (CslText "\x0039")
-    '\x207A' -> CslSup (CslText "\x002B")
-    '\x207B' -> CslSup (CslText "\x2212")
-    '\x207C' -> CslSup (CslText "\x003D")
-    '\x207D' -> CslSup (CslText "\x0028")
-    '\x207E' -> CslSup (CslText "\x0029")
-    '\x207F' -> CslSup (CslText "\x006E")
-    '\x2120' -> CslSup (CslText "\x0053\x004D")
-    '\x2122' -> CslSup (CslText "\x0054\x004D")
-    '\x3192' -> CslSup (CslText "\x4E00")
-    '\x3193' -> CslSup (CslText "\x4E8C")
-    '\x3194' -> CslSup (CslText "\x4E09")
-    '\x3195' -> CslSup (CslText "\x56DB")
-    '\x3196' -> CslSup (CslText "\x4E0A")
-    '\x3197' -> CslSup (CslText "\x4E2D")
-    '\x3198' -> CslSup (CslText "\x4E0B")
-    '\x3199' -> CslSup (CslText "\x7532")
-    '\x319A' -> CslSup (CslText "\x4E59")
-    '\x319B' -> CslSup (CslText "\x4E19")
-    '\x319C' -> CslSup (CslText "\x4E01")
-    '\x319D' -> CslSup (CslText "\x5929")
-    '\x319E' -> CslSup (CslText "\x5730")
-    '\x319F' -> CslSup (CslText "\x4EBA")
-    '\x02C0' -> CslSup (CslText "\x0294")
-    '\x02C1' -> CslSup (CslText "\x0295")
-    '\x06E5' -> CslSup (CslText "\x0648")
-    '\x06E6' -> CslSup (CslText "\x064A")
-    _        -> CslText $ T.singleton c
